import os
import logging
from divinegift.monitoring import send_vk, send_telegram, send_slack, send_email, send_email_with_attachment
import traceback

telegram_chat = {'private': 161680036, 'chat': -277453709, 'channel': -1001343660695}
vk_chat = {'private': '8636128', 'chat': '2000000193'}
to_email_chat = {'private': ['r.rasputin@s7.ru'], 'chat': ['aims.control@s7.ru']}
cc_email_chat = {'private': [], 'chat': []}
log = None


def log_debug(msg):
    logging.debug(msg)
    print(msg)


def log_info(msg):
    logging.info(msg)
    print(msg)


def log_warning(msg):
    logging.warning(msg)
    print(msg)


def log_err(msg, src=None, mode=None, channel='private'):
    logging.exception(msg)
    print(msg)
    error_txt = 'Произошла ошибка в {}\nТекст ошибки: {}\n{}'.format(src, msg, traceback.format_exc())

    #if 'vk' in mode:
    #   send_vk(error_txt, vk_chat[channel.get('vk')], channel.get('vk', 'private'))
    if 'telegram' in mode:
        send_telegram(error_txt, chat_id=telegram_chat[channel.get('telegram', 'private')])
    if 'slack' in mode:
        send_slack()
    if 'email' in mode:
        send_email(src + ' ERROR', error_txt,
                   to_email_chat[channel.get('email', 'private')],
                   cc_email_chat[channel.get('email', 'private')])
    if 'email_attach' in mode:
        send_email_with_attachment(src + ' ERROR', error_txt,
                                   to_email_chat[channel.get('email')],
                                   cc_email_chat[channel.get('email')],
                                   log, os.getcwd() + os.sep)


def log_crit(msg):
    logging.critical(msg)
    print(msg)


def set_loglevel(log_level, log_file):
    global log
    log = log_file
    numeric_level = getattr(logging, log_level.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError('Invalid log level: %s' % log_level)
    print('Add logging with level {} to file {}'.format(log_level, log_file))
    logging.basicConfig(format=u'%(levelname)-8s [%(asctime)s] %(message)s', level=numeric_level,
                        filename=log_file)



from divinegift.logger import log_info
from sqlalchemy import create_engine, MetaData, Table
from string import Template
import json
import sys


class Settings:
    def __init__(self):
        self.settings = {}

    def get_settings(self):
        return self.settings

    def set_settings(self, json_str):
        self.settings = json_str

    def parse_settings(self, file='./settings.conf'):
        json_file = open(file)
        json_str = json_file.read()
        json_data = json.loads(json_str)

        dict_c = dict_compare(self.settings, json_data)
        added, removed, modified, same = dict_c.values()
        if len(added) > 0:
            for r in list(added):
                log_info('Added {}: {}'.format(r, json_data.get(r)))
        if len(removed) > 0:
            for r in list(removed):
                log_info('Removed {}: {}'.format(r, self.settings.get(r)))
        if len(modified) > 0:
            for r in list(modified):
                log_info('Modified {}: {} -> {}'.format(r, modified.get(r)[0], modified.get(r)[1]))

        self.set_settings(json_data)


def dict_compare(d1, d2):
    d1_keys = set(d1.keys())
    d2_keys = set(d2.keys())
    intersect_keys = d1_keys.intersection(d2_keys)
    added = d2_keys - d1_keys
    removed = d1_keys - d2_keys
    modified = {o : (d1[o], d2[o]) for o in intersect_keys if d1[o] != d2[o]}
    same = set(o for o in intersect_keys if d1[o] == d2[o])

    result = {'added': added, 'removed': removed, 'modified': modified, 'same': same}
    return result


def get_conn(db_conn):
    """
    Create connection for SQLAlchemy
    :param db_conn: DB connection (user, password, host, port, db_name)
    :return: Engine, Connection, Metadata
    """
    engine = create_engine('{dialect}://{db_user}:{db_pass}@{db_host}:{db_port}/{db_name}'.format(**db_conn))
    conn = engine.connect()
    metadata = MetaData()
    return engine, conn, metadata


def get_raw_conn(db_conn):
    engine = create_engine('{dialect}://{db_user}:{db_pass}@{db_host}:{db_port}/{db_name}'.format(**db_conn))
    conn = engine.raw_connection()
    return conn


def close_conn(conn):
    conn.close()


def get_sql(filename):
    """
    Get sql string from file
    :param filename: File name
    :return: String with sql
    """
    file = open(filename, 'r')
    sql = file.read()
    file.close()
    return sql


def get_data(sql, db_conn, **kwargs):
    """
    Get raw aims data
    :param sql: File with sql which need to execute
    :param db_conn: DB connect creditions
    :param kwargs: List with additional data
    :return: Dictionary
    """
    #log_info('Get aims data from ' + sql)
    script_t = Template(get_sql(sql))
    script = script_t.safe_substitute(**kwargs)
    #print(script)

    engine, conn, metadata = get_conn(db_conn)
    res = conn.execute(script)
    ress = [dict(row.items()) for row in res]
    close_conn(conn)

    return ress


def run_script(sql, db_conn, **kwargs):
    """
    Run custom script
    :param sql: File with sql which need to execute
    :param db_conn: DB connect creditions
    :param kwargs: List with additional data
    :return: None
    """
    script_t = Template(get_sql(sql))
    script = script_t.safe_substitute(**kwargs)

    engine, conn, metadata = get_conn(db_conn)
    conn.execute(script)
    close_conn(conn)


def get_args():
    args = sys.argv[1:]
    args_d = {}
    for i, arg in enumerate(args[::2]):
        if i == len(args):
            break
        args_d[arg] = args[i * 2 + 1]
    
    return args_d


def get_log_param(args):
    log_level = None
    log_name = None
    for key in args.keys():
        if key in ['--log_level', '-ll']:
            log_level = args.get(key)
        if key in ['--log_name', '-ln']:
            log_name = args.get(key)

    if not log_level:
        log_level = 'INFO'
    if not log_name:
        log_name = None

    return log_level, log_name


def parse_json(json_fname):
    try:
        json_file = open(json_fname)
        json_str = json_file.read()
        json_data = json.loads(json_str)
    except:
        json_data = None

    return json_data


def create_json(json_fname, json_data):
    with open(json_fname, 'w') as outfile:
        json.dump(json_data, outfile)


if __name__ == '__main__':
    pass



import os
import smtplib
import sys
from email import encoders
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.utils import formatdate
import vk_api
import requests


config = {
    'server': 'mail.s7.ru',
    'from': 'noreply@s7.ru'
}


def send_email(subject, body_text, to_emails, cc_emails):
    """
    Send an email
    """

    host = config['server']
    from_addr = config['from']

    BODY = "\r\n".join((
        "From: %s" % from_addr,
        "To: %s" % ', '.join(to_emails),
        "CC: %s" % ', '.join(cc_emails),
        "Subject: %s" % subject,
        "",
        body_text
    ))

    emails = to_emails + cc_emails

    server = smtplib.SMTP(host)
    server.sendmail(from_addr, emails, BODY)
    server.quit()


def send_email_with_attachment(subject, body_text, to_emails, cc_emails, file_to_attach, file_path):
    """
    Send an email with an attachment
    """
    header = 'Content-Disposition', 'attachment; filename="%s"' % file_to_attach

    host = config['server']
    from_addr = config['from']

    # create the message
    msg = MIMEMultipart()
    msg["From"] = from_addr
    msg["Subject"] = subject
    msg["Date"] = formatdate(localtime=True)

    if body_text:
        msg.attach(MIMEText(body_text))

    msg["To"] = ', '.join(to_emails)
    msg["cc"] = ', '.join(cc_emails)

    attachment = MIMEBase('application', "octet-stream")

    try:
        with open(file_path + file_to_attach, "rb") as fh:
            data = fh.read()

        attachment.set_payload(data)
        encoders.encode_base64(attachment)
        attachment.add_header(*header)
        msg.attach(attachment)
    except IOError:
        msg = "Error opening attachment file %s" % file_to_attach
        sys.exit(1)

    emails = to_emails + cc_emails
    server = smtplib.SMTP(host)
    server.sendmail(from_addr, emails, msg.as_string())
    server.quit()


def send_telegram(message, chat_id=161680036, theme=None):
    URL = 'https://api.telegram.org/bot'                        # URL на который отправляется запрос
    TOKEN = '456941934:AAGZMmXJE4VyLagIkVY7qMG0doASxU7f8ac'     # токен вашего бота, полученный от @BotFather
    data = {'chat_id': chat_id,
            'text': (('Тема сообщения: ' + theme + '\n') if theme else '') + 'Сообщение: ' + message}

    try:
        requests.post(URL + TOKEN + '/sendMessage', data=data)  # запрос на отправку сообщения
    except:
        print('Send message error')


def send_slack():
    pass


def auth_vk(login, password):
    # Авторизоваться как человек
    vk = vk_api.VkApi(login=login, password=password)
    vk.auth()
    # Авторизоваться как сообщество
    #vk = vk_api.VkApi(token='a94dd2ef02952a0606fd37f2d1fb11b2d456c034c7671c2b3fab8c3f660474062b9e253c78597d9248469')

    return vk


def send_vk(vk, message, chat_id='8636128', mode='private'):
    #vk = auth_vk()
    if mode == 'private':
        vk.method('messages.send', {'user_id': chat_id, 'message': message})
    elif mode == 'chat':
        vk.method('messages.send', {'peer_id': chat_id, 'message': message})
    elif mode == 'group':
        vk.method('messages.send', {'user_ids': chat_id, 'message': message})


def run():
    aims = {'Я': '8636128', 'Наташа': '39180579', 'Серега': '34123797', 'Паша': '2753320', 'Толя': '406156335'}

    #write_msg_chat('2000000193', 'Проверка отправки сообщений с помощью Python')
    #write_msg(aims['Я'], 'Python шлет тебе привет!')
    send_telegram('Error!')


if __name__ == '__main__':
    run()



from setuptools import setup, find_packages 

setup(name='divinegift',
      version='0.2.3',
      description='It is a Divine Gift',
      long_description='The most useful package for you, young s7_it programer :)',
      classifiers=['Development Status :: 3 - Alpha',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 3',
                   "Operating System :: OS Independent", ],
      keywords='s7_it',
      url='https://github.com/Malanris/DivineGift.git',
      author='Malanris',
      author_email='admin@malanris.ru',
      license='MIT',
      packages=find_packages(),
      install_requires=['sqlalchemy', 'requests', 'vk_api', ],
      include_package_data=True,
      zip_safe=False)